import json
import os
from unittest import TestCase
from unittest import mock
import six

from pago46.client import Pago46


class Pago46TestCase(TestCase):
    def setUp(self):
        """
        Set all enviroment variables to init Pago46 Class
        """
        os.environ["PAGO46_MERCHANT_KEY"] = "secret_merchant_key"
        os.environ["PAGO46_MERCHANT_SECRET"] = "secret_merchant_secret"
        os.environ["PAGO46_API_HOST"] = "https://sandboxapi.pago46.com"

    def test_called_with_all_arguments(self):
        """
        init Pago46 can be done with all it's arguments
        """
        client = Pago46()
        self.assertEqual(client.merchant_key, "secret_merchant_key")
        self.assertEqual(client.merchant_secret, "secret_merchant_secret")
        self.assertEqual(client.pago46_api_host, "https://sandboxapi.pago46.com")

    def test_no_merchant_key_given(self):
        """
        init Pago46 raises an exception if no merchant_key is given
        """
        del os.environ['PAGO46_MERCHANT_KEY']

        six.assertRaisesRegex(self, KeyError, "needs a environment variable called PAGO46_MERCHANT_KEY", Pago46)

    def test_no_merchant_secret_given(self):
        """
        init Pago46 raises an exception if no merchant_secret is given
        """
        del os.environ['PAGO46_MERCHANT_SECRET']

        six.assertRaisesRegex(self, KeyError, "needs a environment variable called PAGO46_MERCHANT_SECRET", Pago46)

    def test_no_pago46_api_host_given(self):
        """
        init Pago46 raises an exception if no pago46_api_host is given
        """
        del os.environ['PAGO46_API_HOST']

        six.assertRaisesRegex(self, KeyError, "needs a environment variable called PAGO46_API_HOST", Pago46)

    @mock.patch('pago46.client.requests.post')
    def test_create_a_new_order(self, mock_post):
        """
        Create a new order on Pago46
        """
        client = Pago46()

        response = {
            "id": "d9125b94-275d-49c9-8a99-ea3511302c9d",
            "price": "4500.00",
            "description": "description",
            "merchant_order_id": "000006",
            "creation_date": "2018-06-27T22:36:56.415643Z",
            "return_url": "http://final.merchant.com",
            "redirect_url": "https://novatest.pago46.com/#UID=d9125b94-275d-49c9-8a99-ea3511302c9d",
            "status": "expired",
            "timeout": 60,
            "notify_url": "http://notification.merchant.com"
        }

        mock_post.return_value = mock.MagicMock(
            headers={'content-type': 'application/json'}, status_code=200, response=json.dumps(response))

        payload = {
            "currency": "CLP",
            "description": "description testing from python librari",
            "merchant_order_id": 'testpythonlibrary1',
            "notify_url": "http://merchant.com/app/response",
            "price": 1000,
            "return_url": "http://final.merchant.com",
            "timeout": 60
        }

        response = client.create_order(payload)
        self.assertEqual(response.status_code, 200)

    @mock.patch('pago46.client.requests.get')
    def test_get_order_by_id(self, mock_get):
        client = Pago46()
        response = {
                    "id": "a6a47f03-d0d3-446b-962d-caf3a7cebb30",
                    "price": "1500.00",
                    "description": "description",
                    "merchant_order_id": "000001",
                    "creation_date": "2018-05-29T20:42:48.853959Z",
                    "return_url": "http://final.merchant.com",
                    "redirect_url": "https://novatest.pago46.com/#UID=a6a47f03-d0d3-446b-962d-caf3a7cebb30",
                    "status": "expired",
                    "timeout": 60,
                    "notify_url": "http://notification.merchant.com"
                }
        mock_get.return_value = mock.MagicMock(
            headers={'content-type': 'application/json'}, status_code=200, response=json.dumps(response))

        order_id = 'a6a47f03-d0d3-446b-962d-caf3a7cebb30'
        response = client.get_order_by_id(order_id)

        self.assertEqual(response.status_code, 200)

    @mock.patch('pago46.client.requests.post')
    def test_mark_order_as_complete(self, mock_post):
        client = Pago46()
        mock_post.return_value = mock.MagicMock(
            headers={'content-type': 'application/json'}, status_code=200, response=json.dumps({'response': 'ok'}))

        payload = {"order_id": "a6a47f03-d0d3-446b-962d-caf3a7cebb30"}
        response = client.mark_order_as_complete(payload)

        self.assertEqual(response.status_code, 200)

    @mock.patch('pago46.client.requests.get')
    def test_get_all_orders(self, mock_get):
        client = Pago46()
        mock_get.return_value = mock.MagicMock(
            headers={'content-type': 'application/json'}, status_code=200, response=json.dumps({'response': 'ok'}))

        response = client.get_all_orders()

        self.assertEqual(response.status_code, 200)

    @mock.patch('pago46.client.requests.get')
    def test_get_order_by_notification_id(self, mock_get):
        client = Pago46()
        mock_get.return_value = mock.MagicMock(
            headers={'content-type': 'application/json'}, status_code=200, response=json.dumps({'response': 'ok'}))
        order_id = 'a6a47f03-d0d3-446b-962d-caf3a7cebb30'
        response = client.get_order_by_notification_id(order_id)

        self.assertEqual(response.status_code, 200)

    @mock.patch('pago46.client.requests.get')
    def test_get_order_details_by_order_id(self, mock_get):
        client = Pago46()
        mock_get.return_value = mock.MagicMock(
            headers={'content-type': 'application/json'}, status_code=200, response=json.dumps({'response': 'ok'}))
        order_id = 'a6a47f03-d0d3-446b-962d-caf3a7cebb30'
        response = client.get_order_details_by_order_id(order_id)

        self.assertEqual(response.status_code, 200)

    @mock.patch('pago46.client.requests.patch')
    def test_update_merchant_order_id(self, mock_patch):
        client = Pago46()
        mock_patch.return_value = mock.MagicMock(
            headers={'content-type': 'application/json'}, status_code=200, response=json.dumps({'response': 'ok'}))
        id = 'a6a47f03-d0d3-446b-962d-caf3a7cebb30'
        new_merchant_order_id = 'merchant_order_edited'
        response = client.update_merchant_order_id(id, new_merchant_order_id)

        self.assertEqual(response.status_code, 200)



import hashlib
import hmac
import time
import urllib.parse
from collections import OrderedDict


def get_concatenated_payload(payload):
    """Concatenate payload"""
    concatenated_data = ''
    params = OrderedDict(sorted(payload.items()))

    for key, value in params.items():
        concatenated_data += "&" + urllib.parse.quote(key, safe='') + "=" + urllib.parse.quote(str(value), safe='')

    return concatenated_data


def sign_request(method, path, merchant_key, merchant_secret, payload={}):
    """sign a request to CPP of PAGO46"""
    date = str(time.time()).replace('.', '')
    encrypt_base = merchant_key + '&' + date + '&'+ method + '&' + urllib.parse.quote(str(path), safe='')
    if len(payload) > 0:
        encrypt_base += get_concatenated_payload(payload)

    message_hash = hmac.new(merchant_secret.encode('utf-8'), encrypt_base.encode('utf-8'),
                            hashlib.sha256).hexdigest()
    return message_hash, date



import os

import requests

from pago46.utils import sign_request


class Pago46(object):
    """Pago46 Client"""
    url_merchant_order = "/merchant/order"
    url_merchant_orders = "/merchant/orders"
    url_merchant_complete = "/merchant/complete/"
    url_merchant_notification = "/merchant/notification"

    def __init__(self):
        """PARAMS to inicialize the client of PAGO46"""
        self.merchant_key = self.__get_merchant_key()
        self.merchant_secret = self.__get_merchant_secret()
        self.pago46_api_host = self.__get_pago46_api_host()

    def __get_merchant_key(self):
        if not "PAGO46_MERCHANT_KEY" in os.environ:
            raise KeyError("needs a environment variable called PAGO46_MERCHANT_KEY")
        return os.environ['PAGO46_MERCHANT_KEY']

    def __get_merchant_secret(self):
        if not "PAGO46_MERCHANT_SECRET" in os.environ:
            raise KeyError("needs a environment variable called PAGO46_MERCHANT_SECRET")
        return os.environ['PAGO46_MERCHANT_SECRET']

    def __get_pago46_api_host(self):
        if not "PAGO46_API_HOST" in os.environ:
            raise KeyError("needs a environment variable called PAGO46_API_HOST")
        return os.environ['PAGO46_API_HOST']

    def get_all_orders(self):
        """ Get all orders from a e-merchant"""
        method = "GET"
        message_hash, date = sign_request(method=method, path=self.url_merchant_orders, merchant_key=self.merchant_key,
                                          merchant_secret=self.merchant_secret)
        headers = {
            "merchant-key": self.merchant_key,
            "message-hash": message_hash,
            "message-date": date
        }
        url = "{}{}".format(self.pago46_api_host, self.url_merchant_orders)
        response = requests.get(url, headers=headers)
        return response

    def create_order(self, payload):
        """Create a order on Pago46
            params:
             -merchant_order_id
             -currency
             -price
             -timeout
             -notify_url
             -return_url
             -description (opcional)
        """
        method = "POST"
        message_hash, date = sign_request(method=method, path=self.url_merchant_orders, payload=payload,
                                          merchant_key=self.merchant_key, merchant_secret=self.merchant_secret)
        headers = {
            "merchant-key": self.merchant_key,
            "message-hash": message_hash,
            "message-date": date
        }
        url = "{}{}".format(self.pago46_api_host, self.url_merchant_orders)
        response = requests.post(url, data=payload, headers=headers)
        return response

    def mark_order_as_complete(self, payload):
        method = "POST"
        message_hash, date = sign_request(method=method, path=self.url_merchant_complete, payload=payload,
                                          merchant_key=self.merchant_key, merchant_secret=self.merchant_secret)
        headers = {
            "merchant-key": self.merchant_key,
            "message-hash": message_hash,
            "message-date": date
        }
        url = "{}{}".format(self.pago46_api_host, self.url_merchant_complete)
        response = requests.post(url, data=payload, headers=headers)
        return response

    def get_order_by_id(self, order_id):
        method = "GET"
        path = "{}/{}".format( self.url_merchant_order, order_id)
        message_hash, date = sign_request(method=method, path=path, merchant_key=self.merchant_key,
                                          merchant_secret=self.merchant_secret)
        headers = {
            "merchant-key": self.merchant_key,
            "message-hash": message_hash,
            "message-date": date
        }
        url = "{}{}/{}".format(self.pago46_api_host, self.url_merchant_order, order_id)

        response = requests.get(url, headers=headers)
        return response

    def get_order_by_notification_id(self, notification_id):
        method = "GET"
        path = "{}/{}".format(self.url_merchant_notification, notification_id)
        message_hash, date = sign_request(method=method, path=path, merchant_key=self.merchant_key,
                                          merchant_secret=self.merchant_secret)
        headers = {
            "merchant-key": self.merchant_key,
            "message-hash": message_hash,
            "message-date": date
        }

        url = "{}{}/{}".format(self.pago46_api_host, self.url_merchant_notification, notification_id)

        response = requests.get(url, headers=headers)
        return response

    def get_order_details_by_order_id(self, order_id):
        method = "GET"
        path = "{}/{}/detail".format(self.url_merchant_order, order_id)
        message_hash, date = sign_request(method=method, path=path, merchant_key=self.merchant_key,
                                          merchant_secret=self.merchant_secret)
        headers = {
            "merchant-key": self.merchant_key,
            "message-hash": message_hash,
            "message-date": date
        }
        url = "{}{}/{}/detail".format(self.pago46_api_host, self.url_merchant_order, order_id)
        response = requests.get(url, headers=headers)
        return response

    def update_merchant_order_id(self, id, new_merchant_order_id):
        method = "PATCH"
        payload = {'merchant_order_id': new_merchant_order_id}

        path = "{}/{}/update/".format(self.url_merchant_order, id)
        message_hash, date = sign_request(method=method, path=path, payload=payload, merchant_key=self.merchant_key,
                                          merchant_secret=self.merchant_secret)
        headers = {
            "merchant-key": self.merchant_key,
            "message-hash": message_hash,
            "message-date": date,
            "Content-type": "application/x-www-form-urlencoded"

        }

        url = "{}{}".format(self.pago46_api_host, path)
        response = requests.patch(url, data=payload, headers=headers)
        return response



"""is the build script for setuptools. It tells setuptools about your """

import setuptools

with open("README.md", "r") as fh:
    LONG_DESCRIPTION = fh.read()

setuptools.setup(
    name="pago46",
    version="0.0.2",
    author="Pago 46",
    author_email="admin@46degrees.net",
    maintainer='David Liencura',
    maintainer_email='dliencura@46degrees.net',
    description="Integraton package of pago46",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url="",
    keywords=['pago46', 'chile', 'payments'],
    install_requires=[
        'requests>=2.19.1',
    ],
    packages=setuptools.find_packages(),
    classifiers=(
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    ),
    license='GPLv3'
)
